/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "help.h"

// Default constructor for the OpenLDev Documentation manager. This function
// initializes all of the objects, sets their properties and creates &
// shows the main window
help::help()
{ 
  // Initialize the objects
  treeview = new Gtk::TreeView();
  swin = new Gtk::ScrolledWindow();
  textview = new Gtk::TextView();
  swin2 = new Gtk::ScrolledWindow();
  hbox = new Gtk::HPaned();
  vbox = new Gtk::VBox(false, 0);
  
  create_toolbar();
  populate_treeview();
  
  // Customize the toolbar
  treeview->signal_button_press_event().connect_notify(sigc::mem_fun(*this, &help::on_treeview));
  swin->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  swin->add(*treeview);
  
  // Customize the display
  textview->set_editable(false);
  textview->set_wrap_mode(Gtk::WRAP_WORD);
  textview->set_left_margin(5);
  swin2->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  swin2->add(*textview);
  
  // Pack the widgets
  hbox->set_position(200);
  hbox->add1(*swin);
  hbox->add2(*swin2);
  vbox->pack_start(*toolbar, Gtk::PACK_SHRINK, 0);
  vbox->pack_start(*hbox);
  
  // Create the window
  set_title("OpenLDev Help");
  set_resizable(true);
  set_size_request(675, 575);
  property_destroy_with_parent().set_value(false);
  add(*vbox);
  show_all_children();
  show();
  open_file();
}

// Create the main toolbar & pack the toolbar buttons into it. This function
// manages the UIManager as well as the actions
void help::create_toolbar()
{
  // Create the toolbar action group
  actgroup = Gtk::ActionGroup::create();
  actgroup->add(Gtk::Action::create("BACK", Gtk::Stock::GO_BACK, "Previous", "Previous"), 
    sigc::mem_fun(*this, &help::on_back));
  actgroup->add(Gtk::Action::create("FORWARD", Gtk::Stock::GO_FORWARD, "Next", "Next"), 
    sigc::mem_fun(*this, &help::on_next));
  actgroup->add(Gtk::Action::create("HOME", Gtk::Stock::HOME, "Index", "Index"), 
    sigc::mem_fun(*this, &help::on_home));
  actgroup->add(Gtk::Action::create("QUIT", Gtk::Stock::QUIT, "Quit", "Quit"), 
    sigc::mem_fun(*this, &help::on_quit));
  
  // Create the UIManager toolbar string
  string ui_info = 
     "<ui>"
     "  <toolbar name='ToolBar'>"
     "    <toolitem action='BACK'/>"
     "    <toolitem action='FORWARD'/>"
     "    <toolitem action='HOME'/>"
     "    <toolitem action='QUIT'/>"
     "  </toolbar>"
     "</ui>";
  
  // Create the UIManager from the string
  uimanager = Gtk::UIManager::create();
  uimanager->insert_action_group(actgroup);
  add_accel_group(uimanager->get_accel_group());
  uimanager->add_ui_from_string(ui_info);
  ui_info = "\0";
  
  // Create the toolbar
  toolbar = dynamic_cast<Gtk::Toolbar*>(uimanager->get_widget("/ToolBar"));
  toolbar->set_toolbar_style(Gtk::TOOLBAR_BOTH_HORIZ);
}

// Populate the treeview. Create the deque of titles of the articles and
// add them to the TreeView.
void help::populate_treeview()
{
  // Populate the titles deque
  title.push_back("OpenLDev Manual");
  title.push_back("Introduction");
  title.push_back(" Menus");
  title.push_back(" Keyboard Shortcuts");
  title.push_back(" Main Window");
  title.push_back("File Operations");
  title.push_back(" Creating a New File");
  title.push_back(" Opening and Saving");
  title.push_back(" Searching for Text");
  title.push_back(" Goto Line in Text");
  title.push_back("Project Management");
  title.push_back(" Creating a New Project");
  title.push_back(" Managing Project Files");
  title.push_back(" Project Settings");
  title.push_back(" Project Information");
  title.push_back("Compiling and Building");
  title.push_back(" Compiling");
  title.push_back(" Building");
  title.push_back(" Auto Generate");
  title.push_back(" Configure");
  title.push_back(" Executing");
  title.push_back(" Other Commands");
  title.push_back("Preferences");
  title.push_back(" Environment Options");
  title.push_back(" Customizing the Toolbar");
  title.push_back("Reporting Bugs");
  title.push_back("About OpenLDev");
  
  // Create the treeview
  Gtk::TreeModel::Row row, child;
  Glib::RefPtr<Gtk::TreeStore> model;
  ModelColumns columns;
  model = Gtk::TreeStore::create(columns);
  treeview->set_model(model);
  treeview->append_column("Help Topics", columns.col_text);
   
  // Loop through the deque of titles
  for (unsigned int i=0; i<title.size(); i++)
  {
    // Add the base row
    row = *(model->append());
    row[columns.col_text] = title.at(i);
    
    // Add all of the base row's children
    for (unsigned int j=i+1; j<title.size(); j++)
    {
      Glib::ustring temp = title.at(j);
      if (temp[0] == ' ')
      {
        temp.erase(0, 1);
        child = *(model->append(row.children()));
        child[columns.col_text] = temp;
        i++;
      }
      else
        break;
    }
  }
  
  treeview->get_selection()->select(treeview->get_model()->get_iter("0"));
}

// Erase the tags from the buffer for header, italic and bold text.
void erase_strings (Glib::RefPtr<Gtk::TextBuffer> buffer)
{
  Glib::ustring strings[6] = {"[h]","[/h]","[i]","[/i]","[b]","[/b]"};
  Gtk::TextIter begin = buffer->begin(), enditer = buffer->end(), end;
  
  // Loop through the text & erase the tags
  for (int i=0; i<6; i++)
  {
    while (begin.forward_search(strings[i],Gtk::TEXT_SEARCH_VISIBLE_ONLY,begin,end,enditer))
    {
      buffer->erase(begin, end);
      begin = buffer->begin();
      enditer = buffer->end();
    }
  }
}

// Open the currently selected file and apply the tags for header,
// italic and bold text.
void help::open_file()
{
  Gtk::TreeModel::iterator iter = treeview->get_selection()->get_selected();
    
  if (iter)
  {
    Gtk::TreeModel::Row row = *iter;
    Glib::ustring fn, file;
    row->get_value(0, fn);
    
    // Erase all of the spaces from the string
    while (fn.find(' ', 0) != string::npos)
      fn.erase(fn.find(' ', 0), 1);
    
    file = "/usr/share/openldev/docs/" + fn;
    
    // Get the text from the file
    Glib::ustring text;
    Glib::RefPtr<Glib::IOChannel> channel = Glib::IOChannel::create_from_file(file, "r");
    channel->read_to_end(text);
    channel->close();
    
    // Set the text to the buffer & refresh the tags
    Glib::RefPtr<Gtk::TextBuffer> buffer = Gtk::TextBuffer::create();
    buffer->set_text(text);
    buffer->remove_all_tags(buffer->begin(), buffer->end());
    
    // Create the TextBuffer tags for bold, header and italic text
    Glib::RefPtr<Gtk::TextBuffer::Tag> bold = Gtk::TextBuffer::Tag::create("Bold");
    bold->property_font().set_value("Sans Bold 10");
    Glib::RefPtr<Gtk::TextBuffer::Tag> header = Gtk::TextBuffer::Tag::create("Header");
    header->property_font().set_value("Sans Bold 14");
    Glib::RefPtr<Gtk::TextBuffer::Tag> italic = Gtk::TextBuffer::Tag::create("Italic");
    italic->property_font().set_value("Sans Italic 10");
    buffer->get_tag_table()->add(bold);
    buffer->get_tag_table()->add(header);
    buffer->get_tag_table()->add(italic);
    
    Gtk::TextIter begin = buffer->begin(), enditer = buffer->end(), iter, end;
    
    // Mark all of the header tags
    while (begin.forward_search("[h]",Gtk::TEXT_SEARCH_VISIBLE_ONLY,begin,end,enditer))
    {
      end = begin;
      end.forward_search("[/h]",Gtk::TEXT_SEARCH_VISIBLE_ONLY,end,iter,enditer);
      buffer->apply_tag(header, begin, end);
      begin++;
    }
    
    // Mark all of the italic tags
    begin = buffer->begin();
    while (begin.forward_search("[i]",Gtk::TEXT_SEARCH_VISIBLE_ONLY,begin,end,enditer))
    {
      end = begin;
      end.forward_search("[/i]",Gtk::TEXT_SEARCH_VISIBLE_ONLY,end,iter,enditer);
      buffer->apply_tag(italic, begin, end);
      begin++;
    }
    
    // Mark all of the bold text
    begin = buffer->begin();
    while (begin.forward_search("[b]",Gtk::TEXT_SEARCH_VISIBLE_ONLY,begin,end,enditer))
    {
      end = begin;
      end.forward_search("[/b]",Gtk::TEXT_SEARCH_VISIBLE_ONLY,end,iter,enditer);
      buffer->apply_tag(bold, begin, end);
      begin++;
    }
    
    // Erase the strings from the buffer & set the buffer to
    // the main TextView
    erase_strings(buffer);
    textview->set_buffer(buffer);
  }
}

// Set the path strings in the array
Glib::ustring paths[] = {"0","1","1:0","1:1","1:2","2","2:0","2:1","2:2","2:3","3","3:0","3:1","3:2","3:3",
 "4","4:0","4:1","4:2","4:3","4:4","4:5","5","5:0","5:1","6","7"};

// Catch a button press event in the TreeView
void help::on_treeview(GdkEventButton *event)
{
  // If the item was double clicked
  if (event->type == GDK_2BUTTON_PRESS)
  {
    Gtk::TreeIter iter = treeview->get_selection()->get_selected();

    // Get the item that was cllicked
    Gtk::TreeModel::Row row = *iter;
    Glib::ustring fn;
    row->get_value(0, fn);
    
    // Expand the path if it is a root tree
    for (unsigned int i=0; i<title.size(); i++)
      if (title.at(i) == fn)
      {
        Gtk::TreePath path(paths[i+1]);
        treeview->expand_to_path(path);
      }   
    
    open_file();
  }
}

// Quit OpenLDev Help
void help::on_quit()
{
  hide();
}

// When the back button is pressed, move to the previous document
void help::on_back()
{
  Gtk::TreeIter iter = treeview->get_selection()->get_selected();
    
  if (iter)
  {
    Gtk::TreeModel::Row row = *iter;
    Glib::ustring fn;
    row->get_value(0, fn);
    int id = 0;
    
    // Loop through & get the id of the previous item
    for (unsigned int i=0; i<title.size(); i++)
    {
      Glib::ustring temp = title.at(i);
      if (temp[0] == ' ')
        temp.erase(0,1);
        
      if (temp == fn)
        id = i - 1;
    }
    
    // If it is not the first item, expand to the previous path &
    // then move to that item
    if (id >= 0)
    {
      Gtk::TreePath path(paths[id]);
      treeview->expand_to_path(path);
      iter = treeview->get_model()->get_iter(paths[id]);
      treeview->get_selection()->select(iter);
      open_file();
    }
  }
}

// When the next button is pressed, move to the next document
void help::on_next()
{
  Gtk::TreeIter iter = treeview->get_selection()->get_selected();
    
  if (iter)
  {
    Gtk::TreeModel::Row row = *iter;
    Glib::ustring fn;
    row->get_value(0, fn);
    unsigned int id = 0;
    
    // Loop through & get the id of the next item
    for (unsigned int i=0; i<title.size(); i++)
    {
      Glib::ustring temp = title.at(i);
      if (temp[0] == ' ')
        temp.erase(0,1);
        
      if (temp == fn)
        id = i + 1;
    }
    
    // If it is not the last item, expand to the next path &
    // then move to that item
    if (id < title.size())
    {
      Gtk::TreePath path(paths[id]);
      treeview->expand_to_path(path);      
      iter = treeview->get_model()->get_iter(paths[id]);
      treeview->get_selection()->select(iter);
      open_file();
    }
  }
}

// Open the first item in the list, the page that gives a description
// of the OpenLDev manual
void help::on_home()
{
  treeview->get_selection()->select(treeview->get_model()->get_iter("0"));
  open_file();
}
