/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "customizetoolbar.h"
#include "utils.h"

// All of the toolbar elements, their id's and their icons
Glib::ustring toolbar_elements[29] = {"-- Separator --","New File","New Project","Open","Save","Save As","Reload File","Quit","Undo","Redo","Cut",
  "Copy","Paste","Find","Find Next","Goto Line","Add Files","Remove File","Project Files","Configure Project","Project Info",
  "Compile","Build","Execute","Auto Generate","Make Clean","Stop","Environment","Help"};
Glib::ustring toolbar_id[29] = {"SP","NF","NP","OP","SV","SA","RV","QT","UD","RD","CT","CO","PT","FD","FN","GL","AF","RF","PF","CP","PI",
  "CL","BU","EX","AG","MC","ST","EN","HP"};
Glib::RefPtr<Gdk::Pixbuf> toolbar_icon[29];

// This function creates all of the widgets and sets their default properties
// for the toolbar customization dialog. It then creates the dialog, shows it
// and connects the buttons to their signals. This receives the environment
// settings class and the current actiongroup
customize_toolbar::customize_toolbar(environment_settings &env, 
                                      Glib::RefPtr<Gtk::ActionGroup> actgroup)
{
  // Initialize all objects
  toolbar = env.get_toolbar();  
  dialog = this; 
  
  // Populate the TreeViews & set selections to the 1st element
  initialize_objects();
  set_icon_array(actgroup);
  populate_available_treeview();
  populate_used_treeview();
  available->get_selection()->select(available->get_model()->get_iter("0"));
  used->get_selection()->select(used->get_model()->get_iter("0"));
  
  dialog->get_action_area()->property_layout_style().set_value(Gtk::BUTTONBOX_END);
  dialog->set_default_icon_from_file(string(INSTALL_PREFIX) + "/share/pixmaps/openldev/openldev24.png");
  
  // Organize the first ScrolledWindow
  swin1->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  swin1->add(*available);

  // Pack the buttons
  vbox->set_border_width(10);
  vbox->pack_start(*add, Gtk::PACK_SHRINK, 5);
  vbox->pack_start(*remove, Gtk::PACK_SHRINK, 5);
  vbox->pack_start(*up, Gtk::PACK_SHRINK, 5);
  vbox->pack_start(*down, Gtk::PACK_SHRINK, 5);
  
  // Organize the secons ScrolledWindow
  swin2->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  swin2->add(*used);
  
  // Pack the ScrolledWindows & the buttons
  hbox->pack_start(*swin1);
  hbox->pack_start(*vbox, Gtk::PACK_SHRINK, 0);
  hbox->pack_start(*swin2);
   
  // Initialize the dialog
  dialog->get_vbox()->set_homogeneous(false);
  dialog->get_vbox()->set_spacing(5);
  dialog->get_vbox()->pack_start(*hbox, Gtk::PACK_EXPAND_WIDGET, 5);
  dialog->set_title("Customize Toolbar");
  dialog->set_size_request(500,375);
  dialog->set_modal(false);
  dialog->property_window_position().set_value(Gtk::WIN_POS_NONE);
  dialog->set_resizable(false);
  dialog->property_destroy_with_parent().set_value(false);
  dialog->set_has_separator(true);
  dialog->add_action_widget(*cancel, -6);
  dialog->add_action_widget(*apply, -10);
  dialog->show_all_children();
  dialog->show();
  
  // Connect the buttons to their signals
  add->signal_clicked().connect(sigc::mem_fun(*this, &customize_toolbar::on_add));
  remove->signal_clicked().connect(sigc::mem_fun(*this, &customize_toolbar::on_delete));
  up->signal_clicked().connect(sigc::mem_fun(*this, &customize_toolbar::on_move_up));
  down->signal_clicked().connect(sigc::mem_fun(*this, &customize_toolbar::on_move_down));
}

// Initialize all of the widgets for the toolbar customization dialog
void customize_toolbar::initialize_objects()
{
  cancel = new Gtk::Button(Gtk::StockID("gtk-cancel"));
  apply = new Gtk::Button(Gtk::StockID("gtk-apply"));
  add = new Gtk::Button(Gtk::StockID("gtk-add"));
  remove = new Gtk::Button(Gtk::StockID("gtk-remove"));
  up = new Gtk::Button(Gtk::StockID("gtk-go-up"));
  down = new Gtk::Button(Gtk::StockID("gtk-go-down"));
  available = new Gtk::TreeView();
  used = new Gtk::TreeView();
  swin1 = new Gtk::ScrolledWindow();
  swin2 = new Gtk::ScrolledWindow();
  vbox = new Gtk::VBox(false, 10);
  hbox = new Gtk::HBox(false, 0);
}

// Load all of the icons for each toolbar item. All of the toolbar elements have
// stock icons except for compile, build and project files items. This function
// receives the actiongroup so the stock icons can be set
void customize_toolbar::set_icon_array(Glib::RefPtr<Gtk::ActionGroup> actgroup)
{
  Glib::RefPtr<Gdk::Colormap> colorm = get_colormap();
  
  // Loop through all of the toolbar options
  for (int i=0; i<29; i++)
  {
    // Load non-stock toolbar element pixbuf(s)
    if (toolbar_id[i] == "CL")
      toolbar_icon[i] = Gdk::Pixbuf::create_from_file(string(INSTALL_PREFIX) + "/share/pixmaps/openldev/compile_sm.png");
    else if (toolbar_id[i] == "BU")
      toolbar_icon[i] = Gdk::Pixbuf::create_from_file(string(INSTALL_PREFIX) + "/share/pixmaps/openldev/build_sm.png");
    else if (toolbar_id[i] == "PF")
      toolbar_icon[i] = Gdk::Pixbuf::create_from_file(string(INSTALL_PREFIX) + "/share/pixmaps/openldev/edit_sm.png");
    // Load stock toolbar elements that are not a separator
    else if (toolbar_id[i] != "SP")
    {
      Gtk::Invisible inv;
      toolbar_icon[i] = inv.render_icon(Gtk::StockID(actgroup->get_action(toolbar_id[i])->property_stock_id().get_value().get_string()), Gtk::IconSize(1));
    }
  }
}

// Populate the TreeView for the available toolbar elements. All of the
// elements are appended to the treeview with their icons.
void customize_toolbar::populate_available_treeview()
{
  ModelColumnsToolbar columns;
  Glib::RefPtr<Gtk::ListStore> model = Gtk::ListStore::create(columns);
  available->set_model(model);
  available->append_column("", columns.col_icon);
  available->append_column("Available Items", columns.col_text);

  // Fill the TreeView's model
  for (int i=0; i<29; i++)
  {
    Gtk::TreeModel::Row row = *(model->append());
    row[columns.col_icon] = toolbar_icon[i];
    row[columns.col_text] = toolbar_elements[i];
  }
}

// Populate the TreeView for the User's Toolbar. After populating the view,
// the number of elements is set for use in other functions.
void customize_toolbar::populate_used_treeview()
{
  ModelColumnsToolbar columns;
  Glib::ustring temp = toolbar, item;
  Glib::RefPtr<Gtk::ListStore> model = Gtk::ListStore::create(columns);
  used->remove_all_columns();
  used->set_model(model);
  used->append_column("", columns.col_icon);
  used->append_column("Toolbar Layout", columns.col_text);

  // Fill the TreeView's model
  int i = 0;
  while (!temp.empty())
  {
    item = temp.substr(0, 2);
    temp.erase(0, 2);
    for (i=0; i<29; i++)
      if (item == toolbar_id[i])
      {
        Gtk::TreeModel::Row row = *(model->append());
        row[columns.col_icon] = toolbar_icon[i];
        row[columns.col_text] = toolbar_elements[i];
      }
  }
  
  // Set the number of items
  max_elements = toolbar.length() / 2 - 1;
}

// The current selection from the available TreeView is added as the first
// element in the User's Toolbar TreeView and it is selected
void customize_toolbar::on_add()
{
  Glib::RefPtr<Gtk::TreeSelection> selection = available->get_selection();
  Gtk::ListStore::iterator iter = selection->get_selected();
  
  if (iter)
  {
    Gtk::TreeModel::Row row = *iter;
    Glib::ustring name;
    row->get_value(1, name);
    
    // Insert the selection
    for (int i=0; i<29; i++)
      if (name == toolbar_elements[i])
        toolbar.insert(0, toolbar_id[i]);
    
    // Update the TreeView
    populate_used_treeview();
    used->get_selection()->select(used->get_model()->get_iter("0"));
    used->scroll_to_row(used->get_model()->get_path(used->get_selection()->get_selected()));
  }
}

// The selected item in the User's Toolbar TreeView is removed and the first
// element in the TreeView is selected
void customize_toolbar::on_delete()
{
  Glib::RefPtr<Gtk::TreeSelection> selection = used->get_selection();
  Gtk::ListStore::iterator iter = selection->get_selected();
  
  // Delete the currently selected item
  if (iter)
  {
    Gtk::TreeModel::Row row = *iter;
    Glib::ustring name, item;
    row->get_value(1, name);
    
    for (int i=0; i<29; i++)
      if (name == toolbar_elements[i])
        item = toolbar_id[i];
    
    // Update the TreeView
    if (toolbar.find(item, 0) != string::npos)
      toolbar.erase(toolbar.find(item, 0), 2);
    populate_used_treeview();
    used->get_selection()->select(used->get_model()->get_iter("0"));
  }
}

// The two characters specified are swapped. The function receives the locations
// of the first and second characters and the string they are contained in. The
// string is passed by reference so all changes are immediately applied
void swap_characters(int id, int id2, Glib::ustring &toolbar)
{
  char ch = toolbar[id], ch2 = toolbar[id2];
  toolbar.erase(id, 1);
  toolbar.insert(id, 1, ch2);
  toolbar.erase(id2, 1);
  toolbar.insert(id2, 1, ch);
}

// The current selection in the User's Toolbar TreeView is moved one space
// up and then it is focused upon unless it is the first element
void customize_toolbar::on_move_up()
{
  Glib::ustring selection = used->get_model()->get_string(used->get_selection()->get_selected());
  locale_from_utf8(selection);
  int id = atol(selection.c_str()) * 2, location = atol(selection.c_str())-1;
  
  if (id != 0)
  {
    // Swap the current & previous items
    swap_characters(id, id-2, toolbar);
    swap_characters(id+1, id-1, toolbar);
    
    // Update the TreeView
    populate_used_treeview();
    gchar line[3];
    snprintf(line, sizeof(line), "%d", location);
    used->get_selection()->select(used->get_model()->get_iter(Glib::ustring(line)));
    used->scroll_to_row(used->get_model()->get_path(used->get_selection()->get_selected()));
  }
}

// The current selection in the User's Toolbar TreeView is moved one space
// down and then it is focused upon unless it is the last element
void customize_toolbar::on_move_down()
{
  Glib::ustring selection = used->get_model()->get_string(used->get_selection()->get_selected());
  locale_from_utf8(selection);
  int id = atol(selection.c_str()) * 2, location = atol(selection.c_str())+1;
  
  if ((location-1) < max_elements)
  {
    // Swap the currently selected & next items
    swap_characters(id+1, id+3, toolbar);
    swap_characters(id, id+2, toolbar);
    
    // Update the TreeView
    populate_used_treeview();
    gchar line[3];
    snprintf(line, sizeof(line), "%d", location);
    used->get_selection()->select(used->get_model()->get_iter(Glib::ustring(line)));
    used->scroll_to_row(used->get_model()->get_path(used->get_selection()->get_selected()));
  }
}

// This allows other classes to get the id string for the toolbar
Glib::ustring customize_toolbar::get_toolbar() { return toolbar; }

// This destructor goes through all of the initialized widgets
// and deletes them to free up memory on close of the dialog
customize_toolbar::~customize_toolbar()
{
  delete cancel;
  delete apply;
  delete add;
  delete remove;
  delete up;
  delete down;
  delete available;
  delete used;
  delete swin1;
  delete swin2;
  delete hbox;
  delete vbox;
}
