/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "project_info.h"
#include "utils.h"

// The project information dialog initializes the object, set their properties,
// creates the dialog & shows it. It receives the project properties object.
project_info::project_info (project_settings proj)
{
  // Initialize the objects
  dialog = this;
  initialize_objects(proj);
   
  close->set_flags(Gtk::CAN_DEFAULT);
  dialog->set_default_icon_from_file(string(INSTALL_PREFIX) + "/share/pixmaps/openldev/openldev24.png");
  dialog->get_action_area()->property_layout_style().set_value(Gtk::BUTTONBOX_END);
  
  // Set alignments for the labels
  label_lines->set_alignment(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER);
  label_files->set_alignment(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER);
  label_lang->set_alignment(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER);
  label_version->set_alignment(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER);
  label_name->set_alignment(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER);
  lines->set_alignment(Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER);
  files->set_alignment(Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER);
  language->set_alignment(Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER);
  version->set_alignment(Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER);
  name->set_alignment(Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER);

  // Attach the labels to the table
  vbox->pack_start(*label_name, Gtk::PACK_SHRINK, 5);
  vbox->pack_start(*label_version, Gtk::PACK_SHRINK, 5);
  vbox->pack_start(*label_lang, Gtk::PACK_SHRINK, 5);
  vbox->pack_start(*label_files, Gtk::PACK_SHRINK, 5);
  vbox->pack_start(*label_lines, Gtk::PACK_SHRINK, 5);
  vbox2->pack_start(*name, Gtk::PACK_SHRINK, 5);
  vbox2->pack_start(*version, Gtk::PACK_SHRINK, 5);
  vbox2->pack_start(*language, Gtk::PACK_SHRINK, 5);
  vbox2->pack_start(*files, Gtk::PACK_SHRINK, 5);
  vbox2->pack_start(*lines, Gtk::PACK_SHRINK, 5);
  hbox->set_spacing(10);
  hbox->set_border_width(10);
  hbox->pack_start(*vbox, Gtk::PACK_SHRINK);
  hbox->pack_start(*vbox2, Gtk::PACK_SHRINK);
  
  // Create the dialog & set the dialog properties
  dialog->get_vbox()->set_homogeneous(false);
  dialog->get_vbox()->set_spacing(0);
  dialog->get_vbox()->pack_start(*hbox);
  dialog->set_title("Project Properties");
  dialog->set_modal(false);
  dialog->property_window_position().set_value(Gtk::WIN_POS_NONE);
  dialog->set_resizable(true);
  dialog->property_destroy_with_parent().set_value(false);
  dialog->set_has_separator(true);
  dialog->add_action_widget(*close, -7);
  dialog->set_default(*close);
  dialog->show_all_children();
  dialog->show();
}

// Initialize all of the widgest for the project information dialog. This
// function receives the project settings object and sets its information to
// the labels.
void project_info::initialize_objects(project_settings proj)
{
  Glib::ustring filenames = proj.get_source_files() + proj.get_header_files();
  
  close = new Gtk::Button(Gtk::StockID("gtk-close"));
  label_lines = new Gtk::Label("Lines of Code:");
  label_files = new Gtk::Label("Source Files:");
  label_lang = new Gtk::Label("Language:");
  label_version = new Gtk::Label("Current Version:");
  label_name = new Gtk::Label("Project Name:");  
  lines = new Gtk::Label(get_lines_of_code(filenames, proj.get_location()));
  files = new Gtk::Label(get_number_of_files(filenames));
  version = new Gtk::Label(proj.get_version());
  name = new Gtk::Label(proj.get_name());
  
  if (proj.get_language() == '+')
    language = new Gtk::Label("C++");
  else
    language = new Gtk::Label("C");
  
  hbox = new Gtk::HBox(false, 0);
  vbox = new Gtk::VBox(false, 0);
  vbox2 = new Gtk::VBox(false, 0);
}

// This destructor goes through all of the initialized widgets
// and deletes them to free up memory on close of the dialog
project_info::~project_info()
{
  delete label_lines;
  delete label_files;
  delete label_name;
  delete label_version;
  delete label_lang;
  delete lines;
  delete files;
  delete name;
  delete version;
  delete language;
  delete close;
  delete table;
}

// Get the number of files in the project. This is done by counting the
// number of backslashes in the files list
Glib::ustring project_info::get_number_of_files (Glib::ustring files)
{
  // Count by how many '/' characters are in the string
  int location = files.find("/", 0), count = 0;
  while (location >= 0)
  {
    count++;
    location = files.find("/", location+1);
  }
  
  // Convert integer properties to character arrays
  gchar buff[5];
  snprintf(buff, sizeof(buff), "%d", count);
  return Glib::ustring(buff);
}

// Get the number of lines of code in the project. The function receives the
// list of filenames and the project source directory location. It then opens
// each of the files & counts how many lines are in the file.
Glib::ustring project_info::get_lines_of_code(Glib::ustring filenames, Glib::ustring location)
{
  // Loop through to get all of the files
  int loc = filenames.find("/",0), count = 0;
  while (loc >= 0) 
  {
    // Create filename & then delete that file from the string
    Glib::ustring fn = location + Glib::ustring("src/") + filenames.substr(0, loc);
    filenames.erase(0, loc+1);
    loc = filenames.find("/",0);
    
    // Get the number of lines in the file
    Glib::ustring text = read_file(fn);
    int loc2 = text.find("\n", 0);
    while (loc2 >= 0)
    {
      count++;
      loc2 = text.find("\n", loc2+1);
    }
    count++;
  }
  
  // Convert integer properties to character arrays
  gchar buff[5];
  snprintf(buff, sizeof(buff), "%d", count);
  return Glib::ustring(buff);
}
